// OPTLAB - Copyright (C) 2018 E. Kuci, C. Geuzaine and P. Duysinx, University of Liege
//
// See the LICENSE.txt file for license information. Please report all
// bugs and problems to the public mailing list <optlab@onelab.info>.

#ifndef _OPTLAB_H_
#define _OPTLAB_H_

// This file defines the OPTLAB C++ API (v1.0).
//
// Do not edit it directly: it is automatically generated by `api/gen.py'.
//
// By design, the OPTLAB C++ API is purely functional, and only uses elementary
// types from the standard library. See `demos/api' for examples.

#if defined(_MSC_VER)
#define _USE_MATH_DEFINES
#endif

#include <cmath>
#include <vector>
#include <string>
#include <utility>

#define OPTLAB_API_VERSION "1.0"

#if defined(OPTLAB_DLL)
#if defined(OPTLAB_DLL_EXPORT)
#define OPTLAB_API __declspec(dllexport)
#else
#define OPTLAB_API __declspec(dllimport)
#endif
#else
#define OPTLAB_API
#endif

namespace optlab {

  // A geometrical entity in the OPTLAB API is represented by two integers: its
  // dimension (dim = 0, 1, 2 or 3) and its tag (its unique, strictly positive
  // identifier). When dealing with multiple geometrical entities of possibly
  // different dimensions, the entities are packed as a vector of (dim, tag)
  // integer pairs.
  typedef std::vector<std::pair<int, int> > vectorpair;

}

namespace optlab { // Top-level functions

  // Initialize SCP.
  OPTLAB_API void initialize(const std::string & type = "none");

  // Solve the optimization problem.
  OPTLAB_API void run();

  // Add the objective function.
  OPTLAB_API void addObjective(const void * func);

  // Solve the optimization problem.
  OPTLAB_API void addConstraint();

  // Solve the optimization problem.
  OPTLAB_API void addGroupOfConstraints();

  // Solve the optimization problem.
  OPTLAB_API void addVariable();

  // Solve the optimization problem.
  OPTLAB_API void addGroupOfVariables();

  // Compute the objective function.
  OPTLAB_API void computeObjective();

  // Finalize Optlab.
  OPTLAB_API void finalize();

  namespace option { // Global option handling functions

    // Set a numerical option to `value'. `name'. Available categories and options
    // are listed in the Scp reference manual.
    OPTLAB_API void setNumber(const std::string & name,
                              const double value);

    // Get the `value' of a numerical option.
    OPTLAB_API void getNumber(const std::string & name,
                              double & value);

    // Print the numerical options.
    OPTLAB_API void show();

  } // namespace option

  namespace mma { // MMA optimizer

    // Initialize MMA. `numberOfConstraints' ... FIXME TODO
    OPTLAB_API void initialize(const std::vector<double> & currentPoint,
                               const std::vector<double> & lowerBound,
                               const std::vector<double> & upperBound,
                               const bool restart = false);

    // Get current point.
    OPTLAB_API void getCurrentPoint(std::vector<double> & currentPoint);

    // Update current point (based on first order derivatives and possibly the
    // diagonal entries of the objective function hessian).
    OPTLAB_API void updateCurrentPoint(const std::vector<double> & constraints,
                                       const std::vector<double> & gradientOfObjective,
                                       const std::vector<double> & gradientOfConstraints,
                                       const std::vector<double> & diagHessianOfObjective = std::vector<double>());

    // Get outer iteration.
    OPTLAB_API void getOuterIteration(int & iteration);

    // Get the infinite norm between two successive designs.
    OPTLAB_API void getDesignChange(double & change);

    // Set the outer move limits.
    OPTLAB_API void setMoveLimits(const std::vector<double> & lowerBound,
                                  const std::vector<double> & upperBound,
                                  const double move);

    // Finalize MMA.
    OPTLAB_API void finalize();

    namespace option { // Global option handling functions

      // Set a numerical option to `value'. `name'. Available categories and
      // options are listed in the Scp reference manual.
      OPTLAB_API void setNumber(const std::string & name,
                                const double value);

      // Get the `value' of a numerical option.
      OPTLAB_API void getNumber(const std::string & name,
                                double & value);

      // Print the numerical options.
      OPTLAB_API void show();

    } // namespace option

  } // namespace mma

} // namespace optlab

#endif
